# -*- coding: utf-8 -*-
# 出退勤のログから日毎の出勤時間を生成するスクリプト
# usage: python merge_log.py attend_log.csv fix_log.yaml > work_log.yaml

import sys
import yaml
import time
import datetime
import settings

HMS = '%H:%M:%S'
HM = '%H:%M'


# スクリプトの実行方法の表示
def print_usage():
    print 'usage: ' + __file__ + ' attend_log.csv fix_log.yaml > work_log.yaml'


def register_work_log(works, date, start_time, end_time, except_hour):
    if start_time == None:
        # 出勤の記録がない日は、前日から働いていたとみなす
        start_time = time.strptime('00:00:00', HMS)

    if end_time == None:
        # 退勤の記録がない日は、翌日まで働いたとみなす
        end_time = time.strptime('23:59:59', HMS)

    works[date] = { 'start_time': start_time,
                    'end_time': end_time,
                    'except_hour': except_hour }

# attend_log.csv ファイルの読み込み
def load_attend_file(file):
    works = {}
    last_date = None
    last_start_time = None
    last_end_time = None
    start_time = None
    end_time = None
    except_hour = settings.except_hour

    for line in open(file, 'r'):
        tokens = line.strip().split(',')
        if len(tokens) != 2:
            continue

        date_time = tokens[0]
        if len(date_time) < (10 + 1 + 8):
            continue

        date, attend_time = date_time.split(' ')
        attend = int(tokens[1])

        # データの日付が変わったときの処理
        if last_date != date:
            # 勤務時間の登録
            if start_time != None:
                register_work_log(works, last_date, start_time,
                                  end_time, except_hour)
            start_time = None
            end_time = None

            # その日の最初のデータの登録
            if attend:
                start_time = time.strptime(attend_time, HMS)
            else:
                end_time = time.strptime(attend_time, HMS)

        else:
            # start, end データの更新
            if attend:
                start_time = min(time.strptime(attend_time, HMS), start_time)
            else:
                end_time = max(time.strptime(attend_time, HMS), end_time)

        last_date = date

    # 最後の日と処理日が一緒だった場合は、データを登録しない
    today = datetime.date.today().strftime('%Y-%m-%d')
    if date != today:
        # 最後の日のデータ登録
        register_work_log(works, date, start_time, end_time, except_hour)

    return works


def fixed_value(token, works, date, field):
    if token == 'None':
        return time.strftime(HM, works[str(date)][field])
    else:
        return token


# fix_log.yaml ファイルの読み込み
def load_fix_file(file, current_works):
    with open(file, 'r') as f:
        f = open(file, 'r')
        data = yaml.load(f)

    if data == None:
        return {}

    works = {}
    for date in sorted(data):
        tokens = data[date]
        start_time = fixed_value(tokens[0], current_works, date, 'start_time')
        end_time = fixed_value(tokens[1], current_works, date, 'end_time')
        except_hour = tokens[2] if len(tokens) >= 3 else settings.except_hour

        register_work_log(works, str(date),
                          time.strptime(start_time, HM),
                          time.strptime(end_time, HM),
                          except_hour)

    return works


def print_work_log(works):
    output = ''
    for key in sorted(works):
        record = works[key]
        output += key + ', ' + \
            time.strftime(HM, record['start_time']) + ', ' + \
            time.strftime(HM, record['end_time']) + ', ' + \
            str(record['except_hour']) + '\n'

    return output


def merged_log(attend_file, fix_file):
    works = load_attend_file(attend_file)
    if fix_file != None:
        works.update(load_fix_file(fix_file, works))
    return print_work_log(works)


if __name__ == '__main__':
    # 引数の解析
    argc = len(sys.argv)
    if argc < 2:
        print_usage()
        sys.exit(1)

    attend_file = sys.argv[1]
    fix_file = sys.argv[2] if argc >= 3 else None

    print merged_log(attend_file, fix_file)
