# -*- coding: utf-8 -*-
# 出退勤のログデータを xls ファイルに流し込むスクリプト
# usage: python register_log.py work_log.csv 2015-07

import sys
import time
import math
from openpyxl import load_workbook
from openpyxl.worksheet.datavalidation import expand_cell_ranges
import settings


# スクリプトの実行方法の表示
def print_usage():
    print 'usage: ' + __file__ + ' work_log.csv 2015-07'


# xls ファイルの更新
def update_cell(sheet, cell, value):
    row = int(cell[1:])
    column = ord(cell[0]) - ord('A') + 1
    cell = sheet.cell(row=row, column=column)
    cell.value = value


def fix_borders(sheet):
    for cell_range in sheet.merged_cell_ranges:
        cells = expand_cell_ranges(cell_range)
        first_cell = cells[0]
        for cell in cells[1:]:
            sheet.cell(cell).border = sheet.cell(first_cell).border


def strip_hour_zero(token):
    if token[0] == '0':
        return token[1:]
    else:
        return token


def hour_float2time(token):
    except_hour = float(token)
    hour = math.floor(except_hour)
    minutes = (except_hour - hour) * 60
    return '%02d:%02d' % (hour, minutes)


# データを挿入した勤務表の出力
def output_roster(work_lines, year_token, month_token):
    book = load_workbook(settings.empty_xls_path)
    sheet = book.get_active_sheet()

    # 名前の更新
    update_cell(sheet, settings.name_cell, settings.name)

    # 年、月の更新
    year = int(year_token)
    month = int(month_token)
    era = (2015 - year) + 27
    update_cell(sheet, settings.year_cell, era)
    update_cell(sheet, settings.month_cell, month)

    # 勤務データの更新
    for line in work_lines.split('\n'):
        tokens = line.strip().split(', ')
        if len(tokens) != 4:
            continue

        y, m, day = tokens[0].split('-')
        if year != int(y) or month != int(m):
            continue

        start_time = strip_hour_zero(tokens[1].strip())
        end_time = strip_hour_zero(tokens[2].strip())
        except_time = strip_hour_zero(hour_float2time(tokens[3]))

        column = settings.insert_column + int(day) - 1
        update_cell(sheet, settings.start_time_row + str(column), start_time)
        update_cell(sheet, settings.end_time_row + str(column), end_time)
        update_cell(sheet, settings.except_hour_row + str(column), except_time)

    # 罫線が Excell で表示されるための処理
    fix_borders(sheet)

    # 必要勤務日数、時間の更新
    year_month = year_token + '-' + month_token
    update_cell(sheet, settings.work_days_cell, settings.work_days[year_month])
    work_hours = settings.work_hour * settings.work_days[year_month]
    update_cell(sheet, settings.work_hours_cell, hour_float2time(work_hours))

    output_file = settings.output_prefix + '_' + \
        settings.name + '_' + year_month + '.xlsx'
    book.save(output_file)


if __name__ == '__main__':
    # 引数の解析
    argc = len(sys.argv)
    if argc < 3:
        print_usage()
        sys.exit(1)

    work_file = sys.argv[1]
    f = open(work_file)
    work_lines = f.readlines()
    f.close()

    year_token, month_token = sys.argv[2].split('-')
    output_roster(work_lines, year_token, month_token)
